from HelperFunctions import *

USER_PATH = '/users'

class User():

	def __init__(self, client):
		self.client = client

	def create_user_path(self, user_id=None):
		if self.client.user_id and not user_id:
			return USER_PATH + '/{0}'.format(self.client.user_id)
		elif user_id and not self.client.user_id:
			return USER_PATH + '/{0}'.format(user_id)
		elif user_id and self.client.user_id:
			return USER_PATH + '/{0}'.format(user_id)
		else:
			return USER_PATH

	'''
		Creates a SynapsePay user and updates the client with the new oauth key.

		:api add_doc		http://api.synapsepay.com/docs/attach-document

		:param user_id		The id of the user to add SSN info to.
		:param options		The body of the API response.

		:return response 	The JSON response
	'''
	def create(self, **kwargs):
		create_keys = ['payload']
		ok, error = checkKwargs(create_keys, kwargs)
		if ok:
			path = self.create_user_path()
			response = self.client.post(path, payload)
			if response.has_key('_id'):
				self.client.set_user_id(response['_id'])
		else:
			response = error
		return analyze_response(response)

	def get(self, **kwargs):
		path = self.create_user_path(kwargs.get('user_id'))
		params={}
		if 'query' in kwargs:
			params['query'] = kwargs.get('query')
		if 'page' in kwargs:
			params['page'] = kwargs.get('page')
		if 'page_count' in kwargs:
			params['per_page'] = kwargs.get('per_page')
		response = self.client.get(path, params)
		if response.has_key('_id'):
			self.client.set_user_id(response['_id'])
		return analyze_response(response)

	def update(self, **kwargs):
		update_keys = ['payload']
		ok, error = checkKwargs(update_keys, kwargs)
		if ok:
			path = self.create_user_path()
			response = self.client.patch(path, kwargs.get('payload'))
		else:
			response = error
		return analyze_response(response)

	def refresh(self, **kwargs):
		refresh_keys = ['payload']
		ok, error = checkKwargs(refresh_keys, kwargs)
		if ok:
			path = '/oauth/{0}'.format(self.client.user_id)
			response = self.client.post(path, kwargs.get('payload'))
			if response.has_key('oauth_key'):
				self.client.update_oauth(response['oauth_key'])
		else:
			response = error
		return analyze_response(response)

	'''
		Adds the SSN information for the specific user. See
		http://api.synapsepay.com/docs/attach-document for more
		detailed explanation of this api and more example payloads.

		:param user_id	The id of the user to add SSN info to.
		:param options	The body of the API response.

		:return response 	The JSON response
	'''
	def add_doc(self, **kwargs):
		update_keys = ['payload']
		ok, error = checkKwargs(update_keys, kwargs)
		if ok:
			path = self.create_user_path()
			response = self.client.patch(path, kwargs.get('payload'))
		else:
			response = error
		return analyze_response(response)

	def verifY(self, **kwargs):
		update_keys = ['payload']
		ok, error = checkKwargs(update_keys, kwargs)
		if ok:
			path = self.create_user_path()
			response = self.client.patch(path, kwargs.get('payload'))
		else:
			response = error
		return analyze_response(response)

	'''
		Uploads a file for the user to help verify their identity. Can be an image, pdf, etc.
		See 
	'''
	def attach_file(self, **kwargs):
		file_keys = ['file']
		ok, error = checkKwargs(file_keys, kwargs)
		if ok:
			base64_image = convert_file_to_base64(kwargs['file'])
			payload = {
				'doc': {
					'attachment':base64_image
				}
			}
			response = self.client.patch(USER_PATH + '/{0}'.format(str(self.client.user_id)), payload)
		else:
			response = error
		return analyze_response(response)


	'''
		Adds the SSN information for the specific user. See
		http://api.synapsepay.com/docs/attach-document for more
		detailed explanation of this api and more example payloads.

		:param user_id	The id of the user to add SSN info to.
		:param options	The body of the API response.

		:return response 	The JSON response
	'''
	def answer_kba(self, payload):
		response = self.client.patch(USER_PATH + '/{0}'.format(str(self.client.user_id)), payload)



import base64
import binascii
import mimetypes


def checkKwargs(keys, kwargs):
	for key in keys:
		if not key in kwargs:
			return False, create_custom_error_message('You are missing the "{0}" parameter'.format(key))
	return True, None

'''
	Converts a file object into a correctly padded base64 representation
	for the SynapsePay API.  Mimetype padding is done by file 
	extension not by content(for now).
'''
def convert_file_to_base64(file_object):
	encoded_string = base64.b64encode(file_object.read())
	mime_type = mimetypes.guess_type(file_object.name)[0]
	mime_padding = 'data:' + mime_type + ';base64,'
	base64_string = mime_padding + encoded_string
	return base64_string

'''
	Used to create JSON formatted error messages that occur at the
	library level.  This allows for no exceptions to ever be raised
	for the user.
'''
def create_custom_error_message(error_message):
	return {
		'success':False,
		'error':{
			'en':error_message
		}
	}

'''
	If we ever wanted to make a different response for the
	user(i.e. just the string message) we could change it here.
	However, for now it just forwards the json response.

	:param response 	The JSON response from the api call.

	:return response 	Returns the JSON response.
'''
def analyze_response(response):
	return response


# Basic wrapper around the the requests library.
from http_client import *
# Assign all the api classes
from api.User import *
from api.Trans import *
from api.Node import *

class Client():

	'''
		Initialize the client to make SynapsePay v3 API calls.

		:param options	{
							'client':{
								'client_id':YOUR_CLIENT_ID,
								'client_secret':YOUR_CLIENT_SECRET
							},
							'user':{
								'fingerprint':USER_FINGERPRINT,
							},
							'oauth':{
								'oauth_key':USER_OAUTH_KEY,
								'refresh_token':USER_REFRESH_TOKEN
							}
						}
	'''
	def __init__(self, options, user_id=None):
		base_url = 'https://synapsepay.com/api/3/users'
		if options.has_key('development_mode'):
			if options['development_mode']:
				base_url = 'https://sandbox.synapsepay.com/api/3/users'

		self.client = HttpClient(options, user_id)
		self.User = User(self.client)
		self.Node = Node(self.client)
		self.Trans = Trans(self.client)



from HelperFunctions import *

NODES_PATH = '/users/{0}/nodes/{1}'

class Node():

	def __init__(self, client):
		self.client = client


	def create_node_path(self, node_id=None):
		if node_id:
			return NODES_PATH.format(self.client.user_id, node_id)
		else:
			return NODES_PATH.replace('/{1}','').format(self.client.user_id)


	def add(self, **kwargs):
		if not 'payload' in kwargs:
			return create_custom_error(message='Set the user id before making this API call.')
		path = self.create_node_path(self.client.user_id)
		response = self.client.post(path, payload)
		return analyze_response(response)


	def verify(self, **kwargs):
		micro_keys = ['payload', 'node_id']
		ok_micro, error_micro = checkKwargs(micro_keys, kwargs)
		mfa_keys = ['payload']
		ok_mfa, error_mfa = checkKwargs(mfa_keys, kwargs)
		if ok_micro:
			path = self.create_node_path(kwargs['node_id'])
		elif ok_mfa:
			path = self.create_node_path()
		else:
			return error_micro
		response = self.client.patch(path, kwargs['payload'])
		return analyze_response(response)	


	def delete(self, **kwargs):
		if not 'node_id' in kwargs:
			return create_custom_error(message='Missing "node_id" argument')
		path = self.create_node_path(kwargs['node_id'])
		response = self.client.delete(path)
		return analyze_response(response)


	def get(self, **kwargs):
		path = None
		if 'node_id' in kwargs:
			path = self.create_node_path(kwargs['node_id'])
		elif self.client.user_id:
			path = self.create_node_path()
		else:
			return create_custom_error(message='Set the user id before making this API call.')
		response = self.client.get(path)
		return analyze_response(response)